// JavaScript code for the BLE Peripheral example app.

var base64 = cordova.require('cordova/base64');

// Application object.
var app = {};

// Device list.
app.devices = {};

// UI methods.
app.ui = {};
// Emulator info
const NAME = "Emulator";
const VERSION = "0.1";
const UUID = "3191-6275-32g4";
var uniHandle;
var uniRequestID;


// Status commands //TODO send en hexadesimal med statuskodene [32,33,34,64,65,66,67,68,71,72,73,74,136]
const  GET_NAME             = 32;
const  GET_VERSION          = 33;
const  GET_UUID             = 34;
const  HAS_LED              = 64;
const  HAS_LED_COLOR        = 65;
const  HAS_VIBRATION        = 66;
const  HAS_COLOR_DETECTION  = 67;
const  HAS_LED_SCREEN       = 68;
const  HAS_RFID             = 71;
const  HAS_NFC              = 72;
const  HAS_ACCELEROMETER    = 73;
const  HAS_TEMPERATURE      = 74;
const  READ_COLOR           = 136;

//Token-solo events
const  TAP				           = 201; //C9
const  DOUBLE_TAP		       = 202; //CA
const  SHAKE				         = 203; //CB
const  TILT                 = 204; //CC

//Token-constraint events
const  MOVE                 = 194; //C2 (+ 2nd and 3rd bytes for current and last sector)

//Token-token events
//TBD

//feedbacks
const  COUNT                = 205; //CD
const  DISPLAY_X            = 206; //CE
const  VIBRATE              = 200; //C8
const  DISPLAY_DIGIT		     = 207;
const  DISPLAY_W		   	     = 208;
const  DISPLAY_UP		       = 209;
const  DISPLAY_DOWN		     = 210;

const  LED_OFF2             = 128;
const  LED_ON2              = 129;
const  LED_BLINK            = 130;


function initialize()
{
	document.addEventListener('deviceready', this.onDeviceReady, false);

}

function onDeviceReady()
{	
	startServer(startAdvert());
}

function startServer(win)
{
	var settings = {
		onConnectionStateChange:function(deviceHandle, connected) {
			hyper.log("device "+deviceHandle+" "+(connected?"":"dis")+"connected");
		},
		services:[
			{
				// matching advert data.
				uuid:"00002220-0000-1000-8000-00805f9b34fb", //0000dead-0000-1000-8000-00805f9b34fb 
				type: 0,
				characteristics:[
					{
						// random uuid.
						uuid:"00002221-0000-1000-8000-00805f9b34fb", 
						handle:1,
						permissions:1|16, //PERMISSION_READ | PERMISSION_WRITE
						properties:2|8|16, // PROPERTY_READ | PROPERTY_WRITE | PROPERTY_NOTIFY  |2| 8| 16
						writeType:2, // WRITE_TYPE_DEFAULT
						onReadRequest:function(deviceHandle, requestId) {
							hyper.log("characteristics 2221 read");
							var data = new Uint8Array([201,1]);
							hyper.log("RequestID: "+requestId+": "+evothings.ble.toUtf8(data));
							evothings.ble.peripheral.sendResponse(deviceHandle, requestId, data, function() {
								hyper.log("cr1 success");
							}, function(err) {
								hyper.log("cr1 fail: "+err);
							});
						},
						onWriteRequest:function(deviceHandle, requestId, data) {
							hyper.log("characteristics 2221 write");
							hyper.log("RequestID: "+requestId+" uft8Data:"+evothings.ble.toUtf8(data));

							evothings.ble.peripheral.sendResponse(deviceHandle, requestId, data, function() {
								hyper.log("cw1 success");
							}, function(err) {
								hyper.log("cw1 fail: "+err);
							});
						},
						descriptors:[
							// notification control.
							(function() {
								hyper.log("Descriptor Notification control");
								var value = new Uint8Array([2,0]);
								return {
									uuid:'00002902-0000-1000-8000-00805f9b34fb',
									permissions:1 | 16,
									onReadRequest:function(deviceHandle, requestId) {
										hyper.log("descriptor 2902 read" + " DEVICEHANDLE: "+ deviceHandle);
										hyper.log("requestId: "+requestId);
										evothings.ble.peripheral.sendResponse(deviceHandle, requestId, value, function() {
											hyper.log("dr1 success");
										}, function(err) {
											hyper.log("dr1 fail: "+err);
										});
									},
									onWriteRequest:function(deviceHandle, requestId, data) {
										hyper.log("descriptor 2902  write" + " DEVICEHANDLE: "+ deviceHandle);
										hyper.log("REQUESTID: " + requestId +" Utf8Data received: "+evothings.ble.toUtf8(data) +" Raw data:"+ data);
										
										uniHandle = deviceHandle;
										uniRequestID = requestId;
										
										evothings.ble.peripheral.sendResponse(deviceHandle, requestId, stringToBytes('null') , function() {
											hyper.log("dw1 success");
										}, function(err) {
											hyper.log("dw1 fail: "+err);
										});
									},
								}
							})(),
							
						],	//descriptors
					},{
						// characteristic uuid receiving commands.
						uuid:"00002222-0000-1000-8000-00805f9b34fb", 
						handle:2,
						permissions:1|16, //PERMISSION_READ | PERMISSION_WRITE
						properties:2|8|16, // PROPERTY_READ | PROPERTY_WRITE | PROPERTY_NOTIFY
						writeType:2, // WRITE_TYPE_DEFAULT
						onReadRequest:function(deviceHandle, requestId) {
							hyper.log("characteristics 2222 read");
							var data = new Uint8Array([201,1]);
							hyper.log("RequestID:  "+requestId+": "+evothings.ble.toUtf8(data));
							
							evothings.ble.peripheral.sendResponse(deviceHandle, requestId, data, function() {
								hyper.log("cr1 success");
							}, function(err) {
								hyper.log("cr1 fail: "+err);
							});
						},
						onWriteRequest:function(deviceHandle, requestId, data) {
							hyper.log("characteristics 2222 write");
							hyper.log("RequestID  "+requestId+":"+evothings.ble.toUtf8(data) + " rawData: " + data);
							uniRequestID+=1;

							evothings.ble.peripheral.sendResponse(deviceHandle, requestId, stringToBytes('null'), function() {
								hyper.log("2222 write success");

							}, function(err) {
								hyper.log("2222 write fail: "+err);
							});
						},
						descriptors:[],
					},
					{
						// characteristic uuid.
						uuid:"00002223-0000-1000-8000-00805f9b34fb", 
						handle:3,
						permissions:1|16, //PERMISSION_READ | PERMISSION_WRITE
						properties:2|8|16, // PROPERTY_READ | PROPERTY_WRITE | PROPERTY_NOTIFY
						writeType:2, // WRITE_TYPE_DEFAULT
						onReadRequest:function(deviceHandle, requestId) {
							hyper.log("characteristics 2223 read");
							var data = new Uint8Array([0,0]);
							hyper.log("RequestID: "+requestId+": "+evothings.ble.toUtf8(data));
							
							evothings.ble.peripheral.sendResponse(deviceHandle, requestId, data, function() {
								hyper.log("cr1 success");
							}, function(err) {
								hyper.log("cr1 fail: "+err);
							});
						},
						onWriteRequest:function(deviceHandle, requestId, data) {
							hyper.log("characteristics 2223 write");
							hyper.log("RequestID: "+requestId+":"+evothings.ble.toUtf8(data));
							
							evothings.ble.peripheral.sendResponse(deviceHandle, requestId, stringToBytes('null'), function() {
								hyper.log("cw1 success");
							}, function(err) {
								hyper.log("cw1 fail: "+err);
							});
						},
						descriptors:[],
					},
					
				],	//characteristics
			},
			
		],	//services
	};

	evothings.ble.peripheral.stopGattServer(function() {
		hyper.log("GATT server already started!?!");
	}, function(err) {
		hyper.log(err+", as expected");
		evothings.ble.peripheral.startGattServer(settings, function() {
			hyper.log("GATT server started successfully.");
			win();
		}, function(err) {
			hyper.log("GATT server start error: "+err);
		});
	});
}

function send_uint8(deviceHandle,requestId,data,length) {
	var sData = {};
	for(var i = 0; i<length; i++) {
		sData[i] = data[i];
	}
	sendResponse(deviceHandle,requestId,sData);
}
function send_string(deviceHandle,requestId,cmd, data) {
	var cmdData = {};
	cmdData[0] = cmd;
	for(var i = 0; i< data.length; i++) {
		cmdData[i+1] = data[i];
	}
	send_uint8(deviceHandle,requestId, cmdData, data.length);
}

function parse(t) {
var sendData ={};
	for(var i = 0; i<t.length; i++ ) {
		sendData[0] = t[i];
		hyper.log("for: " + t[i]);
		switch(t[i]) {
			case GET_NAME:
		      send_string(uniHandle,uniRequestID,GET_NAME, NAME);
		      break;
		    case GET_VERSION:
		      send_string(uniHandle,uniRequestID,GET_VERSION, VERSION);
		      break;
		    case GET_UUID:
		      send_string(uniHandle,uniRequestID,GET_UUID, UUID);
		      break;
		    case HAS_LED:
		      sendData[1] = 1;
		      send_uint8(uniHandle,uniRequestID,sendData, 2);
		      break;
		    case HAS_LED_COLOR:
		      sendData[1] = 1;
		      send_uint8(uniHandle,uniRequestID,sendData, 2);
		      break;
		    case HAS_VIBRATION:
		      sendData[1] = 0;
		      send_uint8(uniHandle,uniRequestID,sendData, 2);
		      break;
		    case HAS_COLOR_DETECTION:
		      sendData[1] = 1;
		      send_uint8(uniHandle,uniRequestID,sendData, 2);
		      break;
		    case HAS_LED_SCREEN:
		      sendData[1] = 0;
		      send_uint8(uniHandle,uniRequestID,sendData, 2);
		      break;
		    case HAS_RFID:
		      sendData[1] = 0;
		      send_uint8(uniHandle,uniRequestID,sendData, 2);
		      break;
		    case HAS_NFC:
		      sendData[1] = 0;
		      send_uint8(uniHandle,uniRequestID,sendData, 2);
		      break;
		    case HAS_ACCELEROMETER:
		      sendData[1] = 0;
		      send_uint8(uniHandle,uniRequestID,sendData, 2);
		      break;
		    case HAS_TEMPERATURE:
		      sendData[1] = 0;
		      send_uint8(uniHandle,uniRequestID,sendData, 2);
		      break;
		    case VIBRATE:
		     // tokenFeedback.vibrate(getData[0] * 10);
		      send_uint8(uniHandle,uniRequestID,sendData, 1);
		      break;
		    case COUNT:
		      //tokenFeedback.displayCount();
		      send_uint8(uniHandle,uniRequestID,sendData, 1);
		      break;
		    case DISPLAY_X:
		      //tokenFeedback.displayX();
		      send_uint8(uniHandle,uniRequestID,sendData, 1);
		      break;
		    case DISPLAY_W:
		     // tokenFeedback.displayW();
		      send_uint8(uniHandle,uniRequestID,sendData, 1);
		      break;
		    case DISPLAY_UP:
		      //tokenFeedback.displayUp();
		      send_uint8(uniHandle,uniRequestID,sendData, 1);
		      break;
		    case DISPLAY_DOWN:
		      //tokenFeedback.displayDown();
		      send_uint8(uniHandle,uniRequestID,sendData, 1);
		      break;
		    case DISPLAY_DIGIT:
		      //tokenFeedback.displayDigit(getData[0]);
		      send_uint8(uniHandle,uniRequestID,sendData, 1);
		      break;
		    default:
		      sendData[0] = 0;
		      send_uint8(uniHandle,uniRequestID,sendData, 1);
		}
	}
}

function sendResponse(deviceHandle,requestId, data) {
	evothings.ble.peripheral.sendResponse(deviceHandle,requestId,data, function() {
		hyper.log("Reponse sent");
	}, function(err) {
		hyper.log("Response Not sent: "+ err);
	});
}

function sendCommand(data) {
	if(uniHandle === '' || uniRequestID === '') {
		return;
	}
	var sendData = stringToBytes(data);
	evothings.ble.peripheral.sendResponse(uniHandle,1, evothings.ble.toUtf8("201,1"), function() {
		hyper.log("Command sent");
	}, function(err) {
		hyper.log("Command not sent: "+ err);
	});

}
// only works for ASCII characters
function bytesToString(buffer) {
    return String.fromCharCode.apply(null, new Uint8Array(buffer));
}

// only works for ASCII characters
function stringToBytes(string) {
    var array = new Uint8Array(string.length);
    for (var i = 0, l = string.length; i < l; i++) {
        array[i] = string.charCodeAt(i);
    }
    return array.buffer;
}


function startAdvert()
{
	stopAdvert();

	var data = {
		includeDeviceName:true,
		includeTxPowerLevel:true,
		//serviceUUIDs:["00001234-0000-1000-8000-00805f9b34fb", "5678abcd-0000-1000-8000-00805f9b34fb"],
		serviceUUIDs:["00002220-0000-1000-8000-00805f9b34fb"],
		//serviceData:{"7d444841-9dc0-11d1-b245-5ffdce74fad2":base64.fromArrayBuffer(new Uint8Array([6,7,8,9,0]))},
		manufacturerData:{"42":base64.fromArrayBuffer(new Uint8Array([1,0]))},
	};

	var scanRespData = {
		//includeDeviceName:false,
		//includeTxPowerLevel:true,
		serviceUUIDs:["deaddead-0000-1000-8000-00805f9b34fb"],
		//serviceData:{"7d444841-9dc0-11d1-b245-5ffdce74fad2":base64.fromArrayBuffer(new Uint8Array([6,7,8,9,0]))},
		serviceData:{"0000dead-0000-1000-8000-00805f9b34fb":base64.fromArrayBuffer(new Uint8Array([1,0]))},
		//manufacturerData:{"42":base64.fromArrayBuffer(new Uint8Array([1,2,3,4,5]))},
	};

	var settings = {
		advertiseMode:"ADVERTISE_MODE_BALANCED",
		connectable:true,
		//timeoutMillis:0,
		txPowerLevel:"ADVERTISE_TX_POWER_HIGH",
		broadcastData:data,
		writeConfigDescriptor: true,
		//scanResponseData:scanRespData,
	};

	evothings.ble.peripheral.startAdvertise(settings,
		function()
		{
			hyper.log("Advertise started.");
		},
		function(errorCode)
		{
			hyper.log("startAdvertise failed: "+errorCode);
		}
	);
}

// Stop scanning for devices.
function stopAdvert()
{
	evothings.ble.peripheral.stopAdvertise();
}